package smart_gs.logical.visitor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import reasoning_web.logical.Connection;
import reasoning_web.logical.inter_face.RWElement;
import smart_gs.connection.LinkChecker;
import smart_gs.logical.GSResource;
import smart_gs.logical.Note;
import smart_gs.logical.Preference;
import smart_gs.logical.Presentation;
import smart_gs.logical.Region;
import smart_gs.logical.Spread;
import smart_gs.logical.SpreadDirectory;
import smart_gs.logical.SpreadDocument;
import smart_gs.logical.StandardTextDocument;
import smart_gs.logical.TextSegment;
import smart_gs.reasoning_web.logical.GSExplanatoryNote;
import smart_gs.reasoning_web.logical.GS_RWElement;
import smart_gs.reasoning_web.logical.GS_RWRepository;

public class CreateXMLElementVisitor implements Visitor {

	private Document document;
	private Stack<Element> elements;

	public CreateXMLElementVisitor(Document document) {
		this.document = document;
		this.elements = new Stack<Element>();
	}

	// public void visit(GSResource resource) {
	// if(resource instanceof Spread){
	// ((Spread)resource).accept(this);
	// }else if(resource instanceof TextSegment){
	// ((TextSegment)resource).accept(this);
	// }else if(resource instanceof Region){
	// ((Region)resource).accept(this);
	// }else if(resource instanceof SpreadDirectory){
	// ((SpreadDirectory)resource).accept(this);
	// }else if(resource instanceof SpreadDocument){
	// ((SpreadDocument)resource).accept(this);
	// }else if(resource instanceof Note){
	// ((Note)resource).accept(this);
	// }else if(resource instanceof StandardTextDocument){
	// ((StandardTextDocument)resource).accept(this);
	// }else if(resource instanceof ExplanatoryNote){
	// ((ExplanatoryNote)resource).accept(this);
	// }
	// }
	public void visit(Spread spread) {
		Element element = document.createElement("spread");
		element.setAttribute(GSResource.NAME, spread.getName());
		element.setAttribute(GSResource.URI, spread.getURI());
		//2007/12/1 kazuhiro kobayashi
//		String filepath = spread.getFile().getAbsolutePath();
		String filepath = spread.getFile().getPath();
		this.isAbsolutePath(filepath, element);
//			element.setAttribute("filename", spread.getFile().getAbsolutePath());
		//

		List<Region> regions = spread.getRegions();

		// ADD northgrid
		// N؂ꔻ
		{
			HashMap<String,String> linkMarge = new HashMap<String,String>();
			HashMap<String,String> linkMap = new HashMap<String,String>();
			for (int i = 0; i < regions.size(); i++) {
				String baseURI = regions.get(i).getBaseURI();
				String uri = regions.get(i).getURI();
				if(baseURI != null){
					linkMarge.put(baseURI, baseURI);
				}
				if(uri != null){
					linkMap.put(uri, uri);
				}
			}
			// RegioñNXg擾
			Iterator<String> itr = linkMarge.keySet().iterator();
			while(itr.hasNext()){
				String baseURI = itr.next();
				ArrayList<Connection> deleteConnection = new ArrayList<Connection>();
				List<Connection> connectList = GS_RWRepository.getInstance().getConnections();
				boolean deleteflg = false;
				for(int i=0;i<connectList.size();i++){
					Connection connection = connectList.get(i);
					RWElement sourceRWElement = connection.getSourcePointer().getElement();
					if(sourceRWElement instanceof GS_RWElement){
						if(((GS_RWElement)sourceRWElement).getURI().startsWith(baseURI)){
							if(!linkMap.containsKey(((GS_RWElement)sourceRWElement).getURI())){
								deleteConnection.add(connection);
								continue;
							}
						}
					}
					RWElement targetRWElement = connection.getTargetPointer().getElement();
					if(targetRWElement instanceof GS_RWElement){
						if(((GS_RWElement)targetRWElement).getURI().startsWith(baseURI)){
							if(!linkMap.containsKey(((GS_RWElement)targetRWElement).getURI())){
								deleteConnection.add(connection);
								continue;
							}
						}
					}
				}
				if(deleteConnection.size() != 0){
					for(int i=0;i<deleteConnection.size();i++){
						Connection connection = deleteConnection.get(i);
						GS_RWRepository.getInstance().removeConnection(connection);
					}
				}
	
			}
		}
		// ADD northgrid

		for (int i = 0; i < regions.size(); i++) {
			regions.get(i).accept(this);
			element.appendChild(this.elements.pop());
		}

		spread.getTranscription().accept(this);
		element.appendChild(this.elements.pop());
		spread.getAnnotation().accept(this);
		element.appendChild(this.elements.pop());
		spread.getTranslation().accept(this);
		element.appendChild(this.elements.pop());

		elements.push(element);
	}

	public void visit(Region region) {
		Element element = document.createElement("region");
		element.setAttribute(GSResource.NAME, region.getName());
		element.setAttribute(GSResource.URI, region.getURI());
		// kazuhiro kobayashi
		element.setAttribute("protection", region.isProtected());
		element.appendChild(region.getView().createXMLElement(document));
		this.elements.push(element);
	}

	public void visit(SpreadDocument doc) {
		Element element = this.document.createElement("spreadDocument");
		element.setAttribute("type", doc.getType() + "");
		element.setAttribute(GSResource.URI, doc.getURI());
		element.setAttribute("leftSource", doc.getLeftPageSource());
		element.setAttribute("rightSource", doc.getRightPageSource());

		//----- NorthGrid
		// N؂𔻒
		ArrayList<Integer> deleteIdList = new ArrayList<Integer>();
		{
			HashMap<String, Integer> linkMap = new HashMap<String, Integer>();;
			try{
				String source = doc.getLeftPageSource();
				source = source.substring(source.indexOf("<xml><left>") + "<xml><left>".length(),source.lastIndexOf("</left></xml>"));
				LinkChecker linkChecker = new LinkChecker();
				linkChecker.parse(source);
				linkMap = linkChecker.getLinkMap();
			}catch(Exception e){}
			// N𔻒
			List<TextSegment> segments = doc.getSegments();
			for (int i = 0; i < segments.size(); i++) {
				int id = segments.get(i).getId();
				if(!linkMap.containsKey(String.valueOf(id))){
					deleteIdList.add(new Integer(id));
				}
			}
		}
		// N폜
		ArrayList<String> deleteUrlList = new ArrayList<String>();
		if(deleteIdList.size() != 0){
			for(int i=0;i<deleteIdList.size();i++){
				Integer id = deleteIdList.get(i);
				TextSegment textSegment = doc.getSegmentByID(id.intValue());
				if(textSegment != null){
					deleteUrlList.add(textSegment.getURI());
					doc.removeTextSegment(id.intValue());
				}
			}
		}
		if(deleteUrlList.size() != 0){
			ArrayList<Connection> deleteConnection = new ArrayList<Connection>();
			List<Connection> connectList = GS_RWRepository.getInstance().getConnections();
			for(int i=0;i<connectList.size();i++){
				Connection connection = connectList.get(i);
				RWElement sourceRWElement = connection.getSourcePointer().getElement();
				if(sourceRWElement instanceof GS_RWElement){
					for(int j=0;j<deleteUrlList.size();j++){
						String url = deleteUrlList.get(j);
						if(url != null){
							if(url.equals(((GS_RWElement)sourceRWElement).getURI())){
								deleteConnection.add(connection);
								break;
							}
						}
					}
				}
				RWElement targetRWElement = connection.getTargetPointer().getElement();
				if(targetRWElement instanceof GS_RWElement){
					for(int j=0;j<deleteUrlList.size();j++){
						String url = deleteUrlList.get(j);
						if(url != null){
							if(url.equals(((GS_RWElement)targetRWElement).getURI())){
								deleteConnection.add(connection);
								break;
							}
						}
					}
				}
			}
			if(deleteConnection.size() != 0){
				for(int i=0;i<deleteConnection.size();i++){
					Connection connection = deleteConnection.get(i);
					GS_RWRepository.getInstance().removeConnection(connection);
				}
			}
		}
		//----- NorthGrid

		List<TextSegment> segments = doc.getSegments();
		for (int i = 0; i < segments.size(); i++) {
			segments.get(i).accept(this);
			element.appendChild(this.elements.pop());
		}
		this.elements.push(element);
	}

	public void visit(TextSegment segment) {
		Element element = this.document.createElement("segment");
		element.setAttribute(GSResource.ID, segment.getId() + "");
		element.setAttribute("tagname", segment.getTagName());
		element.setAttribute(GSResource.URI, segment.getURI());
		this.elements.push(element);
	}

	public void visit(SpreadDirectory directory) {
		Element element = this.document.createElement("directory");
		element.setAttribute(GSResource.NAME, directory.getName());
		element.setAttribute(GSResource.URI, directory.getURI());
		//2007/12/1 kazuhiro kobayashi
//		element.setAttribute("filename", directory.getFile().getAbsolutePath());
//		String filepath = directory.getFile().getAbsolutePath();
		String filepath = directory.getFile().getPath();
		this.isAbsolutePath(filepath, element);
		//
		List<GSResource> resources = directory.getResources();
		for (int i = 0; i < resources.size(); i++) {
			resources.get(i).accept(this);
			element.appendChild(this.elements.pop());
		}
		this.elements.push(element);
	}

	public void visit(Note note) {
		Element element = this.document.createElement("note");
		element.setAttribute(GSResource.NAME, note.getName());
		element.setAttribute(GSResource.URI, note.getURI());
		List<Region> regions = note.getRegions();

		for (int i = 0; i < regions.size(); i++) {
			regions.get(i).accept(this);
			element.appendChild(this.elements.pop());
		}
		this.elements.push(element);
	}

	// 20070923 shimizu wrote;
	public void visit(Presentation present) {
		Element element = this.document.createElement("presentation");
		element.setAttribute("source", present.getSource());
		element.setAttribute(GSResource.NAME, present.getName());
		element.setAttribute(GSResource.ID, present.getId() + "");
	//20071212 shimizu wrote;
		element.setAttribute(GSResource.URI, present.getURI());
		List<TextSegment> segments = present.getSegments();
		for (int i = 0; i < segments.size(); i++) {
			segments.get(i).accept(this);
			element.appendChild(this.elements.pop());
		}
		this.elements.push(element);
	}

	public void visit(StandardTextDocument textDocument) {
		Element element = this.document.createElement("standardDocument");
		element.setAttribute(GSResource.URI, textDocument.getURI());
		element.setAttribute(GSResource.NAME, textDocument.getName());
		element.setAttribute("source", textDocument.getSource());
		List<TextSegment> segments = textDocument.getSegments();
		for (int i = 0; i < segments.size(); i++) {
			segments.get(i).accept(this);
			element.appendChild(this.elements.pop());

		}
		this.elements.push(element);
	}

	public void visit(GSExplanatoryNote note) {
		Element element = this.document.createElement("explanatory");
		element.setAttribute("source", note.getSource());
		element.setAttribute(GSResource.NAME, note.getName());
		element.setAttribute(GSResource.ID, note.getId() + "");
		List<TextSegment> segments = note.getSegments();
		for (int i = 0; i < segments.size(); i++) {
			segments.get(i).accept(this);
			element.appendChild(this.elements.pop());
		}
		this.elements.push(element);

	}

	public Element getLastElement() {
		return this.elements.pop();
	}
	
	//2007/11/30 kazuhiro kobayashi
	private void isAbsolutePath(String filepath, Element element){
		String rootpath = Preference.getInstance().getSmart_gsFolderPath();
		if (filepath.startsWith(rootpath)) {
			/*
			 * "&"xml̃p[Tʂ"&amp;"ɕϊĂ܂肪łȂB
			 * ̂߁AXMLSaverNXŃt@CXML`̕ލۂɒuĂB
			 */
			element.setAttribute("filename", "&rootpath;"
					+ filepath.substring(rootpath.length()));
		} else {
			element.setAttribute("filename", filepath);
		}
	}
}
